@@ -2,10 +2,11 @@ from collections.abc import Sequence
 from typing import TYPE_CHECKING
 
 from BaseClasses import Location, Region, LocationProgressType
-from .data import data, POKEDEX_OFFSET, POKEDEX_COUNT_OFFSET, FLY_UNLOCK_OFFSET, LogicalAccess, GRASS_OFFSET, GrassTile
+from .data import data, LogicalAccess, GrassTile
 from .evolution import evolution_location_name
+from .item_data import POKEDEX_OFFSET, POKEDEX_COUNT_OFFSET, FLY_UNLOCK_OFFSET, GRASS_OFFSET
 from .items import item_const_name_to_id
-from .options import Goal, DexsanityStarters, Grasssanity
+from .options import Goal, DexsanityStarters, Grasssanity, RandomizeBugCatchingContest
 from .pokemon import get_priority_dexsanity, get_excluded_dexsanity
 from .utils import get_fly_regions, get_mart_slot_location_name
 
@@ -15,7 +16,7 @@ if TYPE_CHECKING:
 
 class PokemonCrystalLocation(Location):
     game: str = data.manifest.game
-    rom_address: int | None
+    rom_addresses: list[int]
     default_item_code: int | None
     flag: int | None
     tags: frozenset[str]
@@ -26,14 +27,14 @@ class PokemonCrystalLocation(Location):
             name: str,
             parent: Region | None = None,
             flag: int | None = None,
-            rom_address: int | None = None,
+            rom_addresses: list[int] | None = None,
             default_item_value: int | None = None,
             tags: frozenset[str] = frozenset(),
             progress_type: LocationProgressType = LocationProgressType.DEFAULT
     ) -> None:
         super().__init__(player, name, flag, parent)
         self.default_item_code = default_item_value
-        self.rom_address = rom_address
+        self.rom_addresses = rom_addresses or []
         self.tags = tags
         self.progress_type = progress_type
 
@@ -56,11 +57,23 @@ def create_locations(world: "PokemonCrystalWorld", regions: dict[str, Region]) -
         exclude.add("VanillaClairOn")
     if not world.options.randomize_pokemon_requests:
         exclude.add("BillsGrandpa")
+        exclude.add("PokemonRequest")
     if not world.options.johto_trainersanity and not world.options.kanto_trainersanity:
         exclude.add("Trainersanity")
+    if not world.options.randomize_phone_call_items:
+        exclude.add("Phone Calls")
+
+    exclude.add("Contest")
 
     always_include = {"KeyItem"}
 
+    if world.options.randomize_bug_catching_contest == RandomizeBugCatchingContest.option_all:
+        always_include.add("ContestAll")
+    elif world.options.randomize_bug_catching_contest == RandomizeBugCatchingContest.option_combine_second_third:
+        always_include.add("ContestCombineSecondThird")
+    elif world.options.randomize_bug_catching_contest == RandomizeBugCatchingContest.option_participate:
+        always_include.add("ContestParticipate")
+
     for region_name, region_data in data.regions.items():
         if region_name in regions:
             region = regions[region_name]
@@ -78,17 +91,48 @@ def create_locations(world: "PokemonCrystalWorld", regions: dict[str, Region]) -
                     location_data.label,
                     region,
                     location_data.flag,
-                    location_data.rom_address,
+                    location_data.rom_addresses,
                     location_data.default_item,
                     location_data.tags,
-                    progress_type
+                    progress_type,
                 )
                 region.locations.append(location)
 
+            if world.options.trades_required or world.is_universal_tracker:
+                for trade in region_data.trades:
+                    location = PokemonCrystalLocation(
+                        world.player,
+                        trade,
+                        region,
+                    )
+                    location.show_in_spoiler = False
+                    region.locations.append(location)
+
+            if world.options.goal == Goal.option_unown_hunt:
+                for sign in region_data.signs:
+                    if sign not in world.generated_unown_signs: continue
+                    location = PokemonCrystalLocation(
+                        world.player,
+                        f"{sign}_Encounter",
+                        region
+                    )
+                    location.show_in_spoiler = False
+                    location.place_locked_item(world.create_event("UNOWN"))
+                    region.locations.append(location)
+
+                    location = PokemonCrystalLocation(
+                        world.player,
+                        sign,
+                        region
+                    )
+                    location.show_in_spoiler = False
+                    location.place_locked_item(world.create_event(world.generated_unown_signs[sign]))
+                    region.locations.append(location)
+
     if world.options.dexsanity:
         if not world.is_universal_tracker:
-            pokemon_items = sorted(list(world.logic.available_pokemon))
-            priority_pokemon = sorted(list(get_priority_dexsanity(world)))
+            pokemon_items = sorted(world.logic.available_pokemon)
+            priority_pokemon = sorted(get_priority_dexsanity(world))
             excluded_pokemon = get_excluded_dexsanity(world)
 
             if world.options.dexsanity_starters.value == DexsanityStarters.option_block:
@@ -107,13 +151,13 @@ def create_locations(world: "PokemonCrystalWorld", regions: dict[str, Region]) -
 
         pokedex_region = regions["Pokedex"]
 
-        for pokemon_id in world.generated_dexsanity:
+        for pokemon_id in sorted(world.generated_dexsanity):
             pokemon_data = world.generated_pokemon[pokemon_id]
             new_location = PokemonCrystalLocation(
                 world.player,
                 f"Pokedex - {pokemon_data.friendly_name}",
                 pokedex_region,
-                rom_address=pokemon_data.id,
+                rom_addresses=[pokemon_data.id],
                 flag=POKEDEX_OFFSET + pokemon_data.id,
                 tags=frozenset({"dexsanity"})
             )
@@ -138,7 +182,7 @@ def create_locations(world: "PokemonCrystalWorld", regions: dict[str, Region]) -
                 world.player,
                 f"Pokedex - Catch {dexcountsanity_count} Pokemon",
                 pokedex_region,
-                rom_address=dexcountsanity_count,
+                rom_addresses=[dexcountsanity_count],
                 flag=POKEDEX_COUNT_OFFSET + dexcountsanity_count,
                 tags=frozenset({"dexcountsanity"})
             )
@@ -150,7 +194,7 @@ def create_locations(world: "PokemonCrystalWorld", regions: dict[str, Region]) -
             world.player,
             "Pokedex - Final Catch",
             pokedex_region,
-            rom_address=world.generated_dexcountsanity[-1],
+            rom_addresses=[world.generated_dexcountsanity[-1]],
             flag=POKEDEX_COUNT_OFFSET + len(data.pokemon),
             tags=frozenset({"dexcountsanity"})
         )
@@ -159,7 +203,7 @@ def create_locations(world: "PokemonCrystalWorld", regions: dict[str, Region]) -
     if world.options.evolution_methods_required or world.is_universal_tracker:
         evolution_region = regions["Evolutions"]
         created_locations = set()
-        for pokemon_id, evos_access in world.logic.evolution.items():
+        for pokemon_id, evos_access in sorted(world.logic.evolution.items(), key=lambda x: x[0]):
             for evolution, access in evos_access:
                 if access is LogicalAccess.OutOfLogic and not world.is_universal_tracker: continue
                 location_name = evolution_location_name(world, pokemon_id, evolution.pokemon)
@@ -179,7 +223,7 @@ def create_locations(world: "PokemonCrystalWorld", regions: dict[str, Region]) -
 
     if world.options.breeding_methods_required or world.is_universal_tracker:
         breeding_region = regions["Breeding"]
-        for pokemon_id, children_access in world.logic.breeding.items():
+        for pokemon_id, children_access in sorted(world.logic.breeding.items(), key=lambda x: x[0]):
             accesses = [access for _, access in children_access]
             if LogicalAccess.InLogic not in accesses and not world.is_universal_tracker: continue
             new_location = PokemonCrystalLocation(
@@ -195,7 +239,7 @@ def create_locations(world: "PokemonCrystalWorld", regions: dict[str, Region]) -
             breeding_region.locations.append(new_location)
 
     if world.options.shopsanity:
-        for mart, mart_data in data.marts.items():
+        for mart, mart_data in sorted(data.marts.items(), key=lambda x: x[0]):
             region_name = f"REGION_{mart}"
             if region_name in regions:
                 region = regions[region_name]
@@ -211,7 +255,7 @@ def create_locations(world: "PokemonCrystalWorld", regions: dict[str, Region]) -
                         region,
                         tags=frozenset({"shopsanity"}),
                         flag=item.flag,
-                        rom_address=item.address,
+                        rom_addresses=[item.address],
                         default_item_value=item_const_name_to_id(item.item),
                         progress_type=progress_type
                     )
@@ -229,14 +273,14 @@ def create_locations(world: "PokemonCrystalWorld", regions: dict[str, Region]) -
                 parent_region,
                 tags=frozenset({"fly"}),
                 flag=data.event_flags[f"EVENT_VISITED_{fly_region.base_identifier}"],
-                rom_address=data.rom_addresses[f"AP_FlyUnlock_{fly_region.base_identifier}"],
+                rom_addresses=[data.rom_addresses[f"AP_FlyUnlock_{fly_region.base_identifier}"]],
                 default_item_value=FLY_UNLOCK_OFFSET + fly_region.id
             )
 
             parent_region.locations.append(location)
 
     if world.options.grasssanity == Grasssanity.option_full:
-        for region_id, grass in data.grass_tiles.items():
+        for region_id, grass in sorted(data.grass_tiles.items(), key=lambda x: x[0]):
             if region_id not in regions: continue
             grass_region = regions[f"{region_id}:GRASS"]
 
@@ -246,13 +290,13 @@ def create_locations(world: "PokemonCrystalWorld", regions: dict[str, Region]) -
                         player=world.player,
                         name=grass_tile.name,
                         parent=grass_region,
-                        rom_address=grass_tile.rom_address,
+                        rom_addresses=[grass_tile.rom_address],
                         flag=grass_tile.flag,
                         tags=frozenset({"grass"}),
                     )
                 )
     elif world.options.grasssanity == Grasssanity.option_one_per_area:
-        for location_name, grass_regions in data.grass_regions.items():
+        for location_name, grass_regions in sorted(data.grass_regions.items(), key=lambda x: x[0]):
             region_grass = list[tuple[GrassTile, str]]()
 
             for region in grass_regions:
@@ -275,7 +319,7 @@ def create_locations(world: "PokemonCrystalWorld", regions: dict[str, Region]) -
                 name=location_name,
                 parent=grass_region,
                 flag=flag,
-                rom_address=region_tile.rom_address,
+                rom_addresses=[region_tile.rom_address],
                 tags=frozenset({"grass"}),
             )
             location.original_grass_flag = region_tile.flag
@@ -310,6 +354,18 @@ def create_locations(world: "PokemonCrystalWorld", regions: dict[str, Region]) -
         locs_to_remove = len(trainer_locations) - world.options.kanto_trainersanity.value
         remove_excess_trainersanity(trainer_locations, locs_to_remove)
 
+    if "Bug Catching Contest" in world.options.wild_encounter_methods_required or world.is_universal_tracker:
+        region = regions["REGION_NATIONAL_PARK:CONTEST"]
+
+        for i in range(len(world.generated_contest)):
+            location = PokemonCrystalLocation(
+                player=world.player,
+                name=f"Bug Catching Contest Slot {i + 1}",
+                parent=region,
+            )
+            location.show_in_spoiler = False
+            region.locations.append(location)
+
 
 def create_location_label_to_id_map() -> dict[str, int]:
     """
@@ -368,7 +424,8 @@ LOCATION_GROUPS: dict[str, set[str]] = {
 }
 
 excluded_location_tags = ("VanillaClairOn", "VanillaClairOff", "RequiresSaffronGatehouses", "Badge", "NPCGift",
-                          "Hidden", "KeyItem", "HM", "BillsGrandpa", "BerryTree")
+                          "Hidden", "KeyItem", "HM", "BillsGrandpa", "BerryTree", "ContestAll",
+                          "ContestCombineSecondThird", "PokemonRequest")
 
 for location in data.locations.values():
     for tag in location.tags:
