@@ -9,11 +9,12 @@ import settings
 from BaseClasses import Tutorial, ItemClassification, MultiWorld, CollectionState, Item
 from Fill import fill_restrictive, FillError
 from worlds.AutoWorld import World, WebWorld
-from .breeding import randomize_breeding, generate_breeding_data, can_breed, breeding_is_randomized
+from .breeding import randomize_breeding, can_breed, breeding_is_randomized, get_logically_available_breeding
 from .data import PokemonData, TrainerData, MiscData, TMHMData, data as crystal_data, StaticPokemon, \
-    MusicData, MoveData, FlyRegion, TradeData, MiscOption, POKEDEX_OFFSET, StartingTown, \
-    LogicalAccess, EncounterType, EncounterKey, EncounterMon, EvolutionType, TypeData
-from .evolution import randomize_evolution, generate_evolution_data, evolution_in_logic
+    MusicData, MoveData, FlyRegion, TradeData, MiscOption, StartingTown, LogicalAccess, EncounterType, EncounterKey, \
+    EncounterMon, EvolutionType, TypeData, BugContestEncounter
+from .evolution import randomize_evolution, evolution_in_logic, get_logically_available_evolutions
+from .item_data import POKEDEX_OFFSET
 from .items import PokemonCrystalItem, create_item_label_to_code_map, get_item_classification, ITEM_GROUPS, \
     item_const_name_to_id, item_const_name_to_label, adjust_item_classifications, get_random_filler_item, \
     get_random_ball, place_x_items, PokemonCrystalGlitchedToken
@@ -26,19 +27,21 @@ from .music import randomize_music
 from .options import PokemonCrystalOptions, JohtoOnly, RandomizeBadges, HMBadgeRequirements, FreeFlyLocation, \
     EliteFourRequirement, MtSilverRequirement, RedRequirement, \
     Route44AccessRequirement, RadioTowerRequirement, RequireItemfinder, \
-    OPTION_GROUPS, RandomizeFlyUnlocks, Shopsanity, Grasssanity
+    OPTION_GROUPS, RandomizeFlyUnlocks, Shopsanity, Grasssanity, Goal
 from .phone import generate_phone_traps
 from .phone_data import PhoneScript
-from .pokemon import randomize_pokemon_data, randomize_starters, randomize_traded_pokemon, \
-    fill_wild_encounter_locations, randomize_requested_pokemon
+from .pokemon import randomize_pokemon_data, randomize_starters, fill_wild_encounter_locations, fill_trade_locations, \
+    randomize_unown_signs, randomize_trade_received_pokemon, randomize_trade_requested_pokemon, \
+    get_logically_available_trade_pokemon, randomize_request_pokemon
 from .regions import create_regions, setup_free_fly_regions
 from .rom import generate_output, PokemonCrystalProcedurePatch
 from .rules import set_rules, PokemonCrystalLogic, verify_hm_accessibility
+from .sign_data import FRIENDLY_SIGN_NAMES
 from .trainers import randomize_trainers
 from .universal_tracker import load_ut_slot_data
-from .utils import get_free_fly_locations, randomize_starting_town, \
-    adjust_options
-from .wild import randomize_wild_pokemon, randomize_static_pokemon
+from .utils import get_free_fly_locations, randomize_starting_town, adjust_options
+from .wild import randomize_wild_pokemon, randomize_static_pokemon, get_logically_available_wilds, \
+    get_logically_available_statics
 
 
 class PokemonCrystalSettings(settings.Group):
@@ -47,7 +50,31 @@ class PokemonCrystalSettings(settings.Group):
         copy_to = "Pokemon - Crystal Version (UE) [C][!].gbc"
         md5s = PokemonCrystalProcedurePatch.hash
 
+    class OptionOverrides(dict):
+        """
+        Provided options will be used as overrides when patching.
+        Pass the options as you would in an options yaml. (Option weights and triggers are not supported)
+        Always available option overrides: trainer_name, game_options, field_move_menu_order, default_pokedex_mode
+        Option overrides available when race mode is off: shopsanity_restrict_rare_candies, encounter_slot_distribution, reusable_tms, minimum_catch_rate, skip_elite_four, better_marts, build_a_mart, experience_modifier, starting_money, all_pokemon_seen
+
+        example:
+        option_overrides:
+          experience_modifier: triple
+          trainer_name: CHRIS
+          game_options:
+            turbo_button: a
+            low_hp_beep: off
+        """
+
     rom_file: RomFile = RomFile(RomFile.copy_to)
+    option_overrides = {}
+
+    class MaximumFillerTrapPercentage(int):
+        """
+        The maximum allowed filler_trap_percentage
+        """
+
+    maximum_filler_trap_percentage = MaximumFillerTrapPercentage(20)
 
 
 class PokemonCrystalWebWorld(WebWorld):
@@ -68,7 +95,7 @@ class PokemonCrystalWorld(World):
     Explore the Johto and Kanto regions, become the Pokémon League Champion, and
     defeat the elusive Red at the peak of Mt. Silver!"""
     game = crystal_data.manifest.game
-    apworld_version = crystal_data.manifest.world_version
+    apworld_version = crystal_data.manifest.pokemon_crystal_version
 
     topology_present = True
     web = PokemonCrystalWebWorld()
@@ -106,7 +133,8 @@ class PokemonCrystalWorld(World):
     generated_tms: dict[str, TMHMData]
     generated_wild: dict[EncounterKey, list[EncounterMon]]
     generated_static: dict[EncounterKey, StaticPokemon]
-    generated_trades: list[TradeData]
+    generated_trades: dict[str, TradeData]
+    generated_contest: list[BugContestEncounter]
 
     generated_dexsanity: set[str]
     generated_dexcountsanity: list[int]
@@ -115,6 +143,7 @@ class PokemonCrystalWorld(World):
     generated_starter_helditems: tuple[str, str, str]
     generated_palettes: dict[str, list[int]]
     generated_request_pokemon: list[str]
+    generated_unown_signs: dict[str, str]
 
     generated_music: MusicData
     generated_misc: MiscData
@@ -150,7 +179,8 @@ class PokemonCrystalWorld(World):
         self.generated_tms = dict(crystal_data.tmhm)
         self.generated_wild = {key: list(encounters) for key, encounters in crystal_data.wild.items()}
         self.generated_static = dict(crystal_data.static)
-        self.generated_trades = list(crystal_data.trades)
+        self.generated_trades = dict(crystal_data.trades)
+        self.generated_contest = list(crystal_data.bug_contest_encounters)
         self.generated_dexsanity = set()
         self.generated_dexcountsanity = []
         self.generated_wooper = "WOOPER"
@@ -164,6 +194,7 @@ class PokemonCrystalWorld(World):
         self.generated_misc = replace(crystal_data.misc)
         self.generated_phone_traps = []
         self.generated_phone_indices = []
+        self.generated_unown_signs = {}
 
         self.trainer_name_list = []
         self.trainer_level_list = []
@@ -187,6 +218,7 @@ class PokemonCrystalWorld(World):
     def generate_early(self) -> None:
         adjust_options(self)
         load_ut_slot_data(self)
+        randomize_mischief(self)
         self.logic = PokemonCrystalLogic(self)
 
         if not self.is_universal_tracker:
@@ -199,31 +231,37 @@ class PokemonCrystalWorld(World):
 
             randomize_move_types(self)
             randomize_pokemon_data(self)
+            randomize_unown_signs(self)
 
         self.logic.set_hm_compatible_pokemon(self)
 
     def create_regions(self) -> None:
-        if not self.is_universal_tracker:
-            randomize_starting_town(self)
 
+        randomize_starting_town(self)
         regions = create_regions(self)
 
-        if not self.is_universal_tracker:
-            preevolutions = randomize_evolution(self)
-            randomize_breeding(self, preevolutions)
-            randomize_starters(self)
+        preevolutions = randomize_evolution(self)
+        randomize_breeding(self, preevolutions)
 
+        randomize_starters(self)
         randomize_wild_pokemon(self)
         randomize_static_pokemon(self)
 
+        self.logic.available_pokemon.update(get_logically_available_wilds(self))
+        self.logic.available_pokemon.update(get_logically_available_statics(self))
+
         previous_logically_available_pokemon_count = 0
         while previous_logically_available_pokemon_count != len(self.logic.available_pokemon):
             previous_logically_available_pokemon_count = len(self.logic.available_pokemon)
-            generate_evolution_data(self)
-            generate_breeding_data(self)
+            self.logic.available_pokemon.update(get_logically_available_evolutions(self))
+            self.logic.available_pokemon.update(get_logically_available_breeding(self))
 
-        if not self.is_universal_tracker:
-            randomize_requested_pokemon(self)
+        randomize_trade_requested_pokemon(self)
+        randomize_trade_received_pokemon(self)
+
+        self.logic.available_pokemon.update(get_logically_available_trade_pokemon(self))
+
+        randomize_request_pokemon(self)
 
         create_locations(self, regions)
         self.multiworld.regions.extend(regions.values())
@@ -235,8 +273,6 @@ class PokemonCrystalWorld(World):
 
     def create_items(self) -> None:
 
-        if self.is_universal_tracker: return
-
         item_locations = [
             location
             for location in self.multiworld.get_locations(self.player)
@@ -277,7 +313,10 @@ class PokemonCrystalWorld(World):
             add_items.extend(kanto_badges[:required_badges - 8])
 
         if self.options.johto_only:
-            add_items.append("SUPER_ROD")
+            if self.options.progressive_rods:
+                add_items.append("PROG_ROD")
+            else:
+                add_items.append("SUPER_ROD")
 
         if Shopsanity.blue_card in self.options.shopsanity.value:
             add_items.extend(["BLUE_CARD_PT"] * 5)
@@ -287,18 +326,21 @@ class PokemonCrystalWorld(World):
                 ["RED_APRICORN", "GRN_APRICORN", "BLU_APRICORN", "YLW_APRICORN", "PNK_APRICORN", "BLK_APRICORN",
                  "WHT_APRICORN"])
 
-        trap_names, trap_weights = zip(
-            ("Phone Trap", self.options.phone_trap_weight.value),
-            ("Sleep Trap", self.options.sleep_trap_weight.value),
-            ("Poison Trap", self.options.poison_trap_weight.value),
-            ("Burn Trap", self.options.burn_trap_weight.value),
-            ("Freeze Trap", self.options.freeze_trap_weight.value),
-            ("Paralysis Trap", self.options.paralysis_trap_weight.value),
-        )
-        total_trap_weight = sum(trap_weights)
+        if self.options.goal == Goal.option_unown_hunt:
+            add_items.extend(["KABUTO_TILE"] * 16)
+            add_items.extend(["OMANYTE_TILE"] * 16)
+            add_items.extend(["AERO_TILE"] * 16)
+            add_items.extend(["HO_OH_TILE"] * 16)
 
-        def get_random_trap():
-            return self.create_item(self.random.choices(trap_names, trap_weights)[0])
+        if self.options.add_missing_useful_items:
+            add_items.extend(["TM_9", "TWISTEDSPOON", "THICK_CLUB", "BRIGHTPOWDER", "STICK", "LUCKY_PUNCH",
+                              "LIGHT_BALL", "METAL_POWDER"])
+
+            if Shopsanity.game_corners not in self.options.shopsanity.value:
+                add_items.extend(["TM_14", "TM_15", "TM_25", "TM_32", "TM_38"])
+
+            if self.options.johto_only != JohtoOnly.option_off:
+                add_items.extend(["TM_3", "TM_6", "TM_7", "TM_17", "TM_19", "TM_29", "TM_42", "TM_47"])
 
         for location in item_locations:
             item_code = location.default_item_code
@@ -309,20 +351,17 @@ class PokemonCrystalWorld(World):
 
         if self.options.dexsanity:
             self.itempool.extend(
-                self.create_item_by_const_name(get_random_ball(self.random)) if
-                self.random.randint(0, 100) >= total_trap_weight else get_random_trap()
+                self.create_item_by_const_name(get_random_ball(self.random))
                 for _ in self.generated_dexsanity)
 
         if self.generated_dexcountsanity:
             self.itempool.extend(
-                self.create_item_by_const_name(get_random_ball(self.random)) if
-                self.random.randint(0, 100) >= total_trap_weight else get_random_trap()
+                self.create_item_by_const_name(get_random_ball(self.random))
                 for _ in self.generated_dexcountsanity)
 
         if self.options.grasssanity:
             self.itempool.extend(
-                self.create_item_by_const_name("GRASS_ITEM") if
-                self.random.randint(0, 100) >= total_trap_weight else get_random_trap()
+                self.create_item_by_const_name("GRASS_ITEM")
                 for _ in [loc for loc in self.multiworld.get_locations(self.player) if "grass" in loc.tags])
 
         if self.options.johto_only.value != JohtoOnly.option_off:
@@ -330,6 +369,11 @@ class PokemonCrystalWorld(World):
             self.itempool = [item if item.name != "S.S. Ticket" else self.create_item_by_const_name("SILVER_WING")
                              for item in self.itempool]
 
+        if self.options.progressive_rods:
+            self.itempool = [
+                item if item.name not in ("Old Rod", "Good Rod", "Super Rod") else self.create_item_by_const_name(
+                    "PROG_ROD") for item in self.itempool]
+
         x_items_to_remove = place_x_items(self)
         if x_items_to_remove:
             filtered_itempool = []
@@ -342,12 +386,22 @@ class PokemonCrystalWorld(World):
 
         adjust_item_classifications(self)
 
+        trap_names, trap_weights = zip(
+            ("Phone Trap", self.options.phone_trap_weight.value),
+            ("Sleep Trap", self.options.sleep_trap_weight.value),
+            ("Poison Trap", self.options.poison_trap_weight.value),
+            ("Burn Trap", self.options.burn_trap_weight.value),
+            ("Freeze Trap", self.options.freeze_trap_weight.value),
+            ("Paralysis Trap", self.options.paralysis_trap_weight.value),
+        )
+        total_trap_weight = self.options.filler_trap_percentage.value if any(trap_weights) else 0
+
         for i in range(len(self.itempool)):
             if self.itempool[i].classification == ItemClassification.filler:
                 if add_items:
                     self.itempool[i] = self.create_item_by_const_name(add_items.pop())
-                elif total_trap_weight and self.random.randint(0, 100) < total_trap_weight:
-                    self.itempool[i] = get_random_trap()
+                elif total_trap_weight and self.random.randint(0, 100) <= total_trap_weight:
+                    self.itempool[i] = self.create_item(self.random.choices(trap_names, trap_weights)[0])
 
         adjust_item_classifications(self)
 
@@ -358,6 +412,7 @@ class PokemonCrystalWorld(World):
 
     def generate_basic(self) -> None:
         fill_wild_encounter_locations(self)
+        fill_trade_locations(self)
 
         if self.is_universal_tracker: return
 
@@ -365,9 +420,7 @@ class PokemonCrystalWorld(World):
             verify_hm_accessibility(self)
         randomize_move_values(self)
         cap_hm_move_power(self)
-        randomize_traded_pokemon(self)
         randomize_music(self)
-        randomize_mischief(self)
         randomize_tms(self)
         randomize_type_chart(self)
 
@@ -503,6 +556,7 @@ class PokemonCrystalWorld(World):
             "provide_shop_hints",
             "randomize_fly_unlocks",
             "fly_cheese",
+            "route_42_access",
             "mount_mortar_access",
             "randomize_pokemon_requests",
             "randomize_evolution",
@@ -525,6 +579,18 @@ class PokemonCrystalWorld(World):
             "field_moves_always_usable",
             "grasssanity",
             "enforce_wild_encounter_methods_logic",
+            "randomize_trades",
+            "trades_required",
+            "trap_link",
+            "randomize_bug_catching_contest",
+            "randomize_phone_call_items",
+            "progressive_rods",
+            "add_missing_useful_items",
+            "ss_aqua_access",
+            "magnet_train_access",
+            "route_12_access",
+            "route_30_battle",
+            "require_pokegear_for_phone_numbers",
         )
         slot_data["apworld_version"] = self.apworld_version
         slot_data["tea_north"] = 1 if "North" in self.options.saffron_gatehouse_tea.value else 0
@@ -533,6 +599,7 @@ class PokemonCrystalWorld(World):
         slot_data["tea_west"] = 1 if "West" in self.options.saffron_gatehouse_tea.value else 0
         slot_data["dexsanity_count"] = len(self.generated_dexsanity)
         slot_data["dexsanity_pokemon"] = [self.generated_pokemon[poke].id for poke in self.generated_dexsanity]
+        slot_data["logically_available_pokemon_count"] = len(self.logic.available_pokemon)
 
         region_encounters = dict[str, set[int]]()
         for encounter_key, encounters in self.generated_wild.items():
@@ -544,7 +611,9 @@ class PokemonCrystalWorld(World):
 
         slot_data["region_encounters"] = region_encounters
 
-        for hm in self.options.remove_badge_requirement.valid_keys:
+        slot_data["contest_encounters"] = [self.generated_pokemon[slot.pokemon].id for slot in self.generated_contest]
+
+        for hm in [key for key in self.options.remove_badge_requirement.valid_keys if key not in ("_All", "_Random")]:
             slot_data["free_" + hm.lower()] = 1 if hm in self.options.remove_badge_requirement.value else 0
 
         slot_data["free_fly_location_option"] = self.options.free_fly_location.value
@@ -582,6 +651,8 @@ class PokemonCrystalWorld(World):
             else ool_encounter_method
         slot_data["encmethod_rocksmash"] = 2 if "Rock Smash" in self.options.wild_encounter_methods_required \
             else ool_encounter_method
+        slot_data["encmethod_contest"] = 2 if "Bug Catching Contest" in self.options.wild_encounter_methods_required \
+            else 0
 
         slot_data["evomethod_happiness"] = 1 if "Happiness" in self.options.evolution_methods_required else 0
         slot_data["evomethod_level"] = 1 if "Level" in self.options.evolution_methods_required else 0
@@ -612,7 +683,7 @@ class PokemonCrystalWorld(World):
         slot_data["shopsanity_johtomarts"] = 1 if Shopsanity.johto_marts in self.options.shopsanity.value else 0
         slot_data["shopsanity_kantomarts"] = 1 if Shopsanity.kanto_marts in self.options.shopsanity.value else 0
 
-        evolution_data = dict()
+        evolution_data = dict[int, list[dict]]()
         for pokemon_id, pokemon_data in self.generated_pokemon.items():
             evo_data = list()
             for evo in pokemon_data.evolutions:
@@ -642,6 +713,27 @@ class PokemonCrystalWorld(World):
 
         slot_data["grass_location_mapping"] = self.grass_location_mapping
 
+        slot_data["trades"] = {
+            trade_id: {"requested": str(self.generated_pokemon[trade.requested_pokemon].id), "received": str(
+                self.generated_pokemon[trade.received_pokemon].id)} for trade_id, trade in
+            self.generated_trades.items()}
+
+        slot_data["trap_weights"] = {
+            "Phone Trap": self.options.phone_trap_weight.value,
+            "Sleep Trap": self.options.sleep_trap_weight.value,
+            "Burn Trap": self.options.burn_trap_weight.value,
+            "Freeze Trap": self.options.freeze_trap_weight.value,
+            "Poison Trap": self.options.poison_trap_weight.value,
+            "Paralysis Trap": self.options.paralysis_trap_weight.value,
+        }
+
+        if not self.options.remote_items and self.options.filler_trap_percentage:
+            slot_data["trap_locations"] = {str(location.address): location.item.code for location in
+                                           self.get_locations() if
+                                           location.item.player == self.player and ("Trap" in location.item.tags)}
+
+        slot_data["unown_signs"] = self.generated_unown_signs
+
         return slot_data
 
     def modify_multidata(self, multidata: dict[str, Any]):
@@ -652,6 +744,17 @@ class PokemonCrystalWorld(World):
     def write_spoiler(self, spoiler_handle) -> None:
         spoiler_handle.write(f"\nPokemon Crystal ({self.player_name}):\n")
 
+        if self.options.goal == Goal.option_diploma:
+            available_pokemon = len(self.logic.available_pokemon)
+            spoiler_handle.write(f"Diploma requirement: {available_pokemon}\n species")
+
+        if self.options.goal == Goal.option_unown_hunt:
+            spoiler_handle.write("Unown locations:\n")
+            for sign, unown in self.generated_unown_signs.items():
+                sign_friendly_name = FRIENDLY_SIGN_NAMES[sign]
+                spoiler_handle.write(f"{sign_friendly_name}: {unown.replace('_', ' ')}\n")
+            spoiler_handle.write("\n")
+
         if self.options.randomize_starters:
             spoiler_handle.write("Starters: ")
             spoiler_handle.write(
@@ -679,6 +782,8 @@ class PokemonCrystalWorld(World):
                 for encounter in encounters:
                     if friendly_region_name not in encounters_per_pokemon[encounter.pokemon]:
                         encounters_per_pokemon[encounter.pokemon].append(friendly_region_name)
+            for slot in self.generated_contest:
+                encounters_per_pokemon[slot.pokemon].append("Bug Catching Contest")
         if self.options.randomize_static_pokemon:
             for key, static in self.generated_static.items():
                 if static.level_type == "ignore" or \
@@ -705,25 +810,7 @@ class PokemonCrystalWorld(World):
                 for evo in pokemon_data.evolutions:
                     pokemon_name = self.generated_pokemon[pokemon_id].friendly_name
                     evo_name = self.generated_pokemon[evo.pokemon].friendly_name
-                    evo_type = evo.evo_type
-                    condition = evo.level if evo.evo_type is EvolutionType.Level else evo.condition
-                    if evo_type is EvolutionType.Level:
-                        method = f"Level {condition}"
-                    elif evo_type is EvolutionType.Item:
-                        method = item_const_name_to_label(condition)
-                    elif evo_type is EvolutionType.Happiness:
-                        method = "Happiness"
-                    elif evo_type is EvolutionType.Stats:
-                        if condition == "ATK_GT_DEF":
-                            method = "ATK > DEF"
-                        elif condition == "ATK_LT_DEF":
-                            method = "ATK < DEF"
-                        else:
-                            method = "ATK == DEF"
-                    else:
-                        method = "?"
-
-                    spoiler_handle.write(f"{pokemon_name} -> {method} -> {evo_name}\n")
+                    spoiler_handle.write(f"{pokemon_name} -> {evo.method} -> {evo_name}\n")
 
         if breeding_is_randomized(self):
             spoiler_handle.write(f"\nBreeding ({self.player_name}):\n")
@@ -739,6 +826,13 @@ class PokemonCrystalWorld(World):
                 self.generated_pokemon[pokemon].friendly_name for pokemon in self.generated_request_pokemon)
             spoiler_handle.write(f"\nBill's Grandpa Pokemon ({self.player_name}): {request_pokemon}\n")
 
+        if self.options.randomize_trades:
+            spoiler_handle.write(f"\nTrades ({self.player_name}):\n")
+            for trade in self.generated_trades.values():
+                requested = self.generated_pokemon[trade.requested_pokemon].friendly_name
+                received = self.generated_pokemon[trade.received_pokemon].friendly_name
+                spoiler_handle.write(f"{trade.friendly_name}: {requested} -> {received}\n")
+
         if self.options.grasssanity == Grasssanity.option_one_per_area:
             spoiler_handle.write(f"\nGrass locations ({self.player_name}):\n")
             for loc_id in self.grass_location_mapping.keys():
@@ -750,7 +844,7 @@ class PokemonCrystalWorld(World):
 
     def extend_hint_information(self, hint_data: dict[int, dict[int, str]]):
 
-        def get_dexsanity_wild_hint_data(dexsanity_hint_data: dict[str, list[str]]):
+        def get_dexsanity_wild_hint_data(dexsanity_hint_data: dict[str, set[str]]):
             for key, encounters in self.generated_wild.items():
                 if (self.logic.wild_regions[key] is not LogicalAccess.InLogic) or \
                         (key.encounter_type == EncounterType.Fish and
@@ -766,48 +860,53 @@ class PokemonCrystalWorld(World):
                 for encounter in encounters:
                     if encounter.pokemon not in self.generated_dexsanity:
                         continue
-                    if friendly_region_name not in dexsanity_hint_data[encounter.pokemon]:
-                        dexsanity_hint_data[encounter.pokemon].append(friendly_region_name)
+                    dexsanity_hint_data[encounter.pokemon].add(friendly_region_name)
 
-        def get_dexsanity_static_hint_data(dexsanity_hint_data: dict[str, list[str]]):
+        def get_dexsanity_static_hint_data(dexsanity_hint_data: dict[str, set[str]]):
             for key, static in self.generated_static.items():
-                if static.level_type == "ignore" or key.region_id in ["Entei", "Raikou"]:
+                if static.pokemon not in self.generated_dexsanity or static.level_type == "ignore" or \
+                        key.region_id in ["Entei", "Raikou"]:
                     continue
-                friendly_region_name = key.friendly_region_name()
-                if static.pokemon not in self.generated_dexsanity:
-                    continue
-                if friendly_region_name not in dexsanity_hint_data[static.pokemon]:
-                    dexsanity_hint_data[static.pokemon].append(friendly_region_name)
+                dexsanity_hint_data[static.pokemon].add(key.friendly_region_name())
 
-        def get_dexsanity_evolution_hint_data(dexsanity_hint_data: dict[str, list[str]]):
+        def get_dexsanity_evolution_hint_data(dexsanity_hint_data: dict[str, set[str]]):
             for pokemon_id, pokemon_data in self.generated_pokemon.items():
                 for evo in pokemon_data.evolutions:
-                    if evo.pokemon in self.generated_dexsanity and evolution_in_logic(self, evo):
-                        dexsanity_hint_data[evo.pokemon].append(
-                            f"Evolve {self.generated_pokemon[pokemon_id].friendly_name}")
+                    if not (evo.pokemon in self.generated_dexsanity and evolution_in_logic(self, evo)):
+                        continue
+                    hint_text = f"Evolve {self.generated_pokemon[pokemon_id].friendly_name}"
+                    divergent_evolutions = ("EEVEE", "GLOOM", "POLIWHIRL", "SLOWPOKE", "TYROGUE")
+                    if pokemon_id in divergent_evolutions:
+                        hint_text += f" ({evo.method})"
+                    dexsanity_hint_data[evo.pokemon].add(hint_text)
 
-        def get_dexsanity_breeding_hint_data(dexsanity_hint_data: dict[str, list[str]]):
+        def get_dexsanity_breeding_hint_data(dexsanity_hint_data: dict[str, set[str]]):
             for parent, data in self.generated_pokemon.items():
                 if not can_breed(self, parent): continue
                 child = data.produces_egg
                 parent_name = self.generated_pokemon[parent].friendly_name
                 if child == "NIDORAN_F" and parent != "NIDORAN_M" and "NIDORAN_M" in self.generated_dexsanity:
-                    dexsanity_hint_data["NIDORAN_M"].append(f"Breed {parent_name}")
+                    dexsanity_hint_data["NIDORAN_M"].add(f"Breed {parent_name}")
                 if parent == child: continue
                 if child in self.generated_dexsanity:
-                    dexsanity_hint_data[child].append(f"Breed {parent_name}")
+                    dexsanity_hint_data[child].add(f"Breed {parent_name}")
+
+        def get_dexsanity_trade_hint_data(dexsanity_hint_data: dict[str, set[str]]):
+            for trade in self.generated_trades.values():
+                requested = self.generated_pokemon[trade.requested_pokemon].friendly_name
+                dexsanity_hint_data[trade.received_pokemon].add(f"{trade.friendly_name} - Trade for {requested}")
 
         player_hint_data = dict()
         if self.options.dexsanity:
-            dexsanity_hint_data = defaultdict(list)
-            if self.options.randomize_wilds:
-                get_dexsanity_wild_hint_data(dexsanity_hint_data)
-            if self.options.randomize_static_pokemon and self.options.static_pokemon_required:
+            dexsanity_hint_data = defaultdict(set)
+            get_dexsanity_wild_hint_data(dexsanity_hint_data)
+            if self.options.static_pokemon_required:
                 get_dexsanity_static_hint_data(dexsanity_hint_data)
             if self.options.randomize_evolution:
                 get_dexsanity_evolution_hint_data(dexsanity_hint_data)
             if self.options.breeding_methods_required and breeding_is_randomized(self):
                 get_dexsanity_breeding_hint_data(dexsanity_hint_data)
+            get_dexsanity_trade_hint_data(dexsanity_hint_data)
             player_hint_data |= {
                 self.location_name_to_id[f"Pokedex - {self.generated_pokemon[pokemon_id].friendly_name}"]: ", ".join(
                     methods)
